from components.Timezone import datetime_by_tzone
from enums.ProcessStatusEnum import ProcessStatusEnum
import logging
from typing import Dict, Any, List, Tuple

logger = logging.getLogger()


def get_tasks_from_control(conf: List[Dict[str, Any]], type_task: str) -> Dict[str, Any]:
    response = {'status': ProcessStatusEnum.SUCCESS.value, 'tasks': []}
    try:
        conf = conf[-1]
        logger.info(f"Último proceso ejecutado: {conf}")
        status = conf["status"]
        if status == ProcessStatusEnum.FAIL.value:
            response["status"] = ProcessStatusEnum.FAIL.value
            success_tasks = conf["tasks"]
            final_tasks = []
            for key in success_tasks.keys():
                if type_task.lower() not in key.lower():
                    continue
                task = success_tasks[key]
                task_status = task["status"]
                if task_status == ProcessStatusEnum.SUCCESS.value:
                    command = task["description"]
                    result = (key, command)
                    final_tasks.append(result)
            response["tasks"] = final_tasks
    except Exception as e:
        logger.error(f"Error obteniendo task fallidos desde control. {e}")
    finally:
        return response


def update_new_process(conf: List[Dict[str, Any]], status: str, tasks: Dict[str, Any],
                       timezone: str, delete_last: bool = False) -> List[Dict[str, Any]]:
    try:
        if delete_last:
            print("CONF-TRANS:", conf)
            last_tasks = conf[-1]["tasks"]
            tasks.update(last_tasks)
            print("LAST:", tasks)
            conf.pop(-1)
        current_datetime = str(datetime_by_tzone(timezone, '%Y-%m-%d %H:%M:%S'))
        new_process = {"date": current_datetime, "status": status, "tasks": tasks}
        conf.append(new_process)
    except Exception as e:
        logger.error(f"Error actualizando archivo de control. {e}")
    finally:
        return conf
